/*
 * Run a program residing in the filesystem.
 *
 * Copyright (C) 2022, 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/sys/err.h>

#include <fsclient/client.h>
#include <fsclient/process.h>
#include <systypes/fcntl.h>
#include <systypes/format.h>

#include <stdio.h>

#include "ops.h"



/* Transfer size for communication. */

static const offset_t TO_TRANSFER = 1024;



/* Run the given program. */

int run_program(int argc, char *argv[])
{
  char buffer[TO_TRANSFER];
  process_t *process;
  file_t *reader, *writer;
  notify_flags_t flags;
  notify_values_t values;
  long err;

  /* Create a pipe for process output. */

  err = client_pipe(&reader, &writer, 0);

  if (err)
  {
    printf("Could not obtain pipe: %s\n", l4sys_errtostr(err));
    return -1;
  }

  /* Start the process. */

  err = process_spawn(argc, (const char **) argv, writer, &process);

  if (err)
  {
    printf("Could not start process: %s\n", l4sys_errtostr(err));
    return -1;
  }

  printf("Finished program initiation.\n");

  /* Release the writing end of the pipe. */

  client_close(writer);

  /* Read until the pipe yields no more data. */

  while (1)
  {
    offset_t nread = client_read(reader, buffer, TO_TRANSFER);

    if (nread)
      fwrite(buffer, sizeof(char), nread, stdout);
    else
      break;
  }

  /* Close the pipe and obtain the process state. */

  client_close(reader);

  err = process_wait(process, &flags, &values);

  printf("End process (flags %" pFMTnotify_flags "x values: %ld, %ld)\n", flags, values.sig, values.val);

  if (err)
    return -1;

  return values.val;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
